<?php
// command.php
// created the 08/12/05 by mickael ROYER

require_once('mpd-class/mpd.class.php');
require_once('config/config.php');
require_once('include/common.php');
require_once('include/block.php');
require_once('include/cls/class.auth.php');
require_once('include/cls/class.config.php');
require_once('include/auth.php');
// domxml extension does not work with php5
if (version_compare(PHP_VERSION,'5','>='))
  require_once('include/cls/dom-php4-to-php5.php');

// Webradios playlist name
define('WEBRADIOS_PLAYLIST_NAME','__webradios__');
define('CURRENT_PLAYLIST_NAME','__current__');

session_start();

// load the configuration
$config =& init_config();

// Active authentification if necessary
if (isset($auth_config)) {
  $auth =& init_auth();
  $auth->check();
}

/*
 * function to send queue command
*/
function send_queue_command($myMpd,$xml_root,$command,$objects,$decode = TRUE) {

  foreach ($objects as $object) {
    $value = ($decode ? format_text($object) : $object);

    if (is_null($myMpd->QueueCommand($command,$value))) { 
      xml_message($xml_root,t("Unable to add a command in the queue :").$myMpd->errStr,'error');
      return false;
      }
    }

  if (is_null($myMpd->SendCommandQueue())) {
    xml_message($xml_root,t("Unable to send queued commands ").$myMpd->command_queue.' : '.
      $myMpd->errStr,'error');
    return false;
    }

  return true;
}

/*
 * functions used to construct the xml file sent at the web browser
*/

function xml_tag($type,$xml_root,$nodes) {
  $xml_type = $xml_root->new_child($type,'');

  // First, Create a virtual tag corresponding at all the tag
  // It is not a really good solution, but i do not find better for the moment
  $xml_type->new_child('node','<b>'.t('All').'</b>');

  foreach ($nodes as $id => $value)
  	$xml_type->new_child('node',htmlspecialchars($value,ENT_QUOTES));
}

function xml_tag_navigation($xml_root,$myMpd,$artist = NULL,$album =NULL) {
  $artists = $myMpd->GetArtists();
  $albums = $myMpd->GetAlbums();
  if (is_null($artists)) { 
    xml_message($xml_root,t("Unable to get the list of artists : ").$myMpd->errStr,'error');
    return;
    }
  elseif (is_null($albums)) {
    xml_message($xml_root,t("Unable to get the list of albums : ").$myMpd->errStr,'error');
    return;
    }

  $xml_tagnav = $xml_root->new_child('library','');
  $xml_tagnav->set_attribute('type','tag');

  xml_tag('artists',$xml_tagnav,$artists);
  xml_tag('albums',$xml_tagnav,$albums);
}

function xml_track($xml,$id,$entry, $current_track = FALSE) {
  if ($current_track)
    $track = $xml->new_child('current_track','');
  else
    $track = $xml->new_child('track','');
  $track->new_child('num',$id);

  $displayed_time = NULL;
  if( array_key_exists('Time',$entry) )
    $displayed_time = $entry['Time'];
  else
    $displayed_time = 0;
  $track->new_child('time',$displayed_time);

  if (array_key_exists('Artist',$entry) && $entry['Artist'])
    $track->new_child('artist',htmlspecialchars($entry['Artist']));
  if (array_key_exists('Album',$entry) && $entry['Album'])
    $track->new_child('album',htmlspecialchars($entry['Album']));

  if (array_key_exists('Title',$entry) && $entry['Title'])
    $track->new_child('title',htmlspecialchars($entry['Title']));
  elseif (array_key_exists('file',$entry) && $entry['file']) {
    $tmp = explode("/", $entry['file']);
    $title = $tmp[0] != 'http:' ? $tmp[count($tmp) -'1'] : $entry['file']; // webradios case
    $track->new_child('title',htmlspecialchars($title));
    }
}

function xml_webradios_track($playlist,$id,$entry) {
    $track = $playlist->new_child('track','');
    $track->new_child('num',$id);
    $track->new_child('title',htmlspecialchars($entry['file']));
}

function xml_playlist($xml_root,$myMpd,$webradios = false) {
  if (is_null($myMpd->playlist)) {
    xml_message($xml_root,t("Problem with the playlist :").$myMpd->errStr,'error');
    }
  else {
    $playlist = $xml_root->new_child('playlist','');
    if ($webradios) {
      $playlist->set_attribute('webradios',1);
      foreach ($myMpd->playlist as $id => $entry) {
        xml_webradios_track($playlist,$id,$entry);
        }
      }
    else {
      foreach ($myMpd->playlist as $id => $entry) {
        xml_track($playlist,$id,$entry);
        }
      }
    }
}

function xml_control($xml_root,$myMpd) {
  $xml_control = $xml_root->new_child('control','');
// obtain the state of the server playing, stopped, repeat random ...
  $state = $myMpd->state;
  $xml_control->new_child('state',$state);
  $xml_control->new_child('random',$myMpd->random);
  $xml_control->new_child('repeat',$myMpd->repeat);
  $xml_control->new_child('xfade',$myMpd->xfade);

  if ($state == MPD_STATE_PLAYING || $state == MPD_STATE_PAUSED) {
  // return the current track
    if (($id = $myMpd->current_track_id)== -1) {
      }
    else { 
      xml_track($xml_control,$id,$myMpd->playlist[$id],TRUE);
  // return the current_track_position and the current_track_length
      if (is_null($curtrackpos = $myMpd->current_track_position) || is_null($curtracklen = $myMpd->current_track_length)) 
         xml_message($xml_root,t("Unable to get the current_track: ").$myMpd->errStr,'error');
      else {
        $xml_control->new_child('current_track_position',$curtrackpos);
        $xml_control->new_child('current_track_length',$curtracklen);
        }
      }
    }
}

function xml_volume($xml_root,$myMpd) {
  if (is_null($vol = $myMpd->volume))
    xml_message($xml_root,t("Unable to get the volume: ").$myMpd->errStr,'error');
  else
    $xml_root->new_child('volume',$vol);
}

function xml_playlist_dir($xml_root,$myMpd) {
  $playlist_dir = $xml_root->new_child('library','');
  $playlist_dir->set_attribute('type','playlist');
  if (is_null($playlist_list = $myMpd->GetDir()))
    xml_message($xml_root,t("Unable to get the playlist list: ").$myMpd->errStr,'error');
  else {
    foreach($playlist_list as $id => $entry) {
      if (array_key_exists("playlist",$entry) &&
          $entry["playlist"] != WEBRADIOS_PLAYLIST_NAME &&
	  $entry["playlist"] != CURRENT_PLAYLIST_NAME) {
        $pl_file = $playlist_dir->new_child('track_node',htmlspecialchars($entry["playlist"],ENT_QUOTES));
  	$pl_file->set_attribute("type","playlist");
	}
      }
    }
}

function xml_webradios_dir($xml_root,$myMpd) {

  // First we save the current playlist
  $myMpd->PLDelete(CURRENT_PLAYLIST_NAME);
  if (is_null($myMpd->PLSave(CURRENT_PLAYLIST_NAME)))
    xml_message($xml_root,t("Unable to save the current playlist : ").$myMpd->errStr,'error');

  if (!is_null($myMpd->PLClear()) && 
	is_null($myMpd->PLLoad(WEBRADIOS_PLAYLIST_NAME))) {

    // Try to create the webradios special playlist if it does not exist
    if ( is_null($myMpd->PLSave(WEBRADIOS_PLAYLIST_NAME)) )
      xml_message($xml_root,t("Unable to load the webradios : ").$myMpd->errStr,'error');

  }

  $pl = $xml_root->new_child('block',webradios_playlist());
  $pl->set_attribute('id','playlist');
  $pl = $xml_root->new_child('block',webradios_playlist_extra());
  $pl->set_attribute('id','playlist_extra');
  xml_playlist($xml_root,$myMpd,true);
}

function xml_nav_dir($xml_root,$myMpd,$path = '') {
  $dir = $xml_root->new_child('library','');
  $dir->set_attribute('type','directory');
  if (is_null($list_dir = $myMpd->GetDir($path)))
    xml_message($xml_root,t("Unable to get the directories list: ").$myMpd->errStr,'error');
  else {
    foreach($list_dir as $id => $entry) { 
      if (array_key_exists("directory",$entry)) { 
        $file = $dir->new_child('track_node',htmlspecialchars($entry["directory"],ENT_QUOTES));
  	$file->set_attribute("type","directory");
	}
      elseif (array_key_exists("file",$entry)) {
        $file = $dir->new_child('track_node',htmlspecialchars($entry["file"],ENT_QUOTES));
  	$file->set_attribute("type","file");
	}
      }
    if ($path)
      $dir->set_attribute("path",str_replace("\'","'",htmlspecialchars($path)));
    }
}

function xml_config($xml_root) {
 global $config; 
 $config_value = array('lang','theme','enable_shortcut_keys','refresh','message_time','request_timeout','dbl_click');
 $xml_config = $xml_root->new_child('config','');

 foreach($config_value as $value)
   $xml_config->new_child($value,$config->variable_get($value));
}

function xml_message($xml_root,$message,$type) {
  $xml_message = $xml_root->new_child('message',$message);
  $xml_message->set_attribute("type",$type);
}

function xml_track_info($xml_file,$entry) {
  $xml_file->new_child('artist',$entry['Artist']);
  $xml_file->new_child('album',$entry['Album']);
  $xml_file->new_child('title',$entry['Title']);
  $xml_file->new_child('path',$entry['file']);
}

function xml_l10n($xml_root) {
  $msgs = array('playlist_notrack_warning' => t('You must have track in the playlist to record it.'),
  		'playlist_noname_warning' => t('You must enter a name to record the playlist.'),
  		'no_track_playing' => t('No track playing'),
  		'no_playlist_found' => t('No playlist found'),
  		'request_timeout' => t('Request Timed Out!'),
  		'confirm_plerase' => t('Are you sure you want to delete selected playlists')
  		);

  $xml_l10n = $xml_root->new_child('l10n','');
  foreach ($msgs as $name => $msg) {
    $xml_l10n_msg = $xml_l10n->new_child('l10n_msg',$msg);
    $xml_l10n_msg->set_attribute("name",$name);
    }
}

function xml_form_errors($xml_root,$form_errors,$form_name) {
  $xml_errors = $xml_root->new_child('form_errors','');
  $xml_errors->set_attribute('name',$form_name);

  foreach($form_errors as $el_name => $el_error) {
    $xml_error = $xml_errors->new_child('error',$el_error);
    $xml_error->set_attribute('id',$el_name);
    }
}

/*********************************************************************
 Main function
**********************************************************************/

// initialisation of the object which control mpd server
if ($mpd_config['password']) {
  $myMpd = new mpd($mpd_config['host'],$mpd_config['port'],$mpd_config['password']);
  }
else {
  $myMpd = new mpd($mpd_config['host'],$mpd_config['port']);
  }

// initialisation of xml document
$xml = domxml_new_doc('1.0');

$xml_root = $xml->add_root('mpd');

// connection at the server
if ( $myMpd->connected == FALSE ) {
  xml_message($xml_root,t("Problem to connect at mpd : ").$myMpd->errStr,'error');
  }
else {
  $action = $_GET['action'];
  switch ($action) {

// Command Part
    case 'next' :
      if (is_null($myMpd->Next()))
        xml_message($xml_root,t("Unable to jump at the next track: ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;

    case 'previous' :
      if (is_null($myMpd->Previous()))
        xml_message($xml_root,t("Unable to jump at the previous track: ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;

    case 'play' :
      if (is_null($myMpd->Play()))
        xml_message($xml_root,t("Unable to play song: ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;

    case 'skipto' :
      if (is_null($myMpd->SkipTo($_GET['id'])))
        xml_message($xml_root,t("Unable to play the select song: ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;

    case 'stop' :
      if (is_null($myMpd->Stop()))
        xml_message($xml_root,t("Unable to stop the current track: ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;

    case 'pause' :
      if (is_null($myMpd->Pause()))
        xml_message($xml_root,t("Unable to pause the current track: ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;

    case 'random' :
      $new_value = ($myMpd->random ? '0' : '1');
      if (is_null($myMpd->SetRandom($new_value)))
        xml_message($xml_root,t("Unable to set the random mode : ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;

    case 'repeat' :
      $new_value = ($myMpd->repeat ? '0' : '1');
      if (is_null($myMpd->SetRepeat($new_value)))
        xml_message($xml_root,t("Unable to set the repeat mode : ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;


// control part
    case 'get_control' :
      xml_control($xml_root,$myMpd);
      break;

    case 'set_track_position' : 
      $value = 0;
      if (isset($_GET['value'])) 
        $value = floor(($_GET['value']/'100')*($myMpd->current_track_length)); 
      if (is_null($myMpd->SeekTo($value)))
        xml_message($xml_root,t("Unable to set the new track position: ").$myMpd->errStr,'error');
      xml_control($xml_root,$myMpd);
      break;

// Volume Part
    case 'set_volume' : 
      $vol = 0;
      if (isset($_GET['vol']))
        $vol = $_GET['vol']; 
      if (is_null($myMpd->SetVolume($vol)))
        xml_message($xml_root,t("Unable to set the volume: ").$myMpd->errStr,'error');
      xml_volume($xml_root,$myMpd);
      break;

    case 'get_volume' : 
      xml_volume($xml_root,$myMpd);
      break;

// Playlist Part
    case 'get_playlist' :
      xml_playlist($xml_root,$myMpd);
      xml_control($xml_root,$myMpd);
      break;

    case 'clear_playlist' :
      if (is_null($myMpd->PLClear())) 
        xml_message($xml_root,t("Unable to clear the playlist: ").$myMpd->errStr,'error');
      xml_playlist($xml_root,$myMpd);
      xml_control($xml_root,$myMpd);
      break;

    case 'remove_playlist' :
      $ids = $_POST['edit'];
      rsort($ids);

      if (!send_queue_command($myMpd,$xml_root,MPD_CMD_PLREMOVE,$ids,false))
      	break;

      // FIXME : there are a problem with the refresh after the SendCommandQueue
      // Must to be investigate
      //xml_control($xml_root,$myMpd);
      //xml_playlist($xml_root,$myMpd);
      $xml_root->new_child('javascript',"ajaxmp_ref.send_http_request('GET','command.php?action=get_playlist','',true);");
      break;

    case 'add_playlist' :
      $files = $_POST['edit'];

      if (!send_queue_command($myMpd,$xml_root,MPD_CMD_PLADD,$files,TRUE))
      	break;

      // FIXME : there are a problem with the refresh after the SendCommandQueue
      // Must to be investigate
      //xml_playlist($xml_root,$myMpd);
      //xml_control($xml_root,$myMpd);
      $xml_root->new_child('javascript',"ajaxmp_ref.send_http_request('GET','command.php?action=get_playlist','',true);");
      break;

    case 'add_webradio' :
      $files = $_POST['edit'];

      if (!send_queue_command($myMpd,$xml_root,MPD_CMD_PLADD,$files,TRUE))
      	break;

      if (is_null($myMpd->PLDelete(WEBRADIOS_PLAYLIST_NAME)) ||
          is_null($myMpd->PLSave(WEBRADIOS_PLAYLIST_NAME)) )
        xml_message($xml_root,t("Unable to add webradio: ").$myMpd->errStr,'error');

      // FIXME : there are a problem with the refresh after the SendCommandQueue
      // Must to be investigate
      $xml_root->new_child('javascript',"ajaxmp_ref.send_http_request('GET','command.php?action=get_webradios','',true);");
      break;

    case 'clear_webradio' :
      if (is_null($myMpd->PLClear())) 
        xml_message($xml_root,t("Unable to erase webradios : ").$myMpd->errStr,'error');

      if (is_null($myMpd->PLDelete(WEBRADIOS_PLAYLIST_NAME)) ||
          is_null($myMpd->PLSave(WEBRADIOS_PLAYLIST_NAME)) )
        xml_message($xml_root,t("Unable to erase webradios : ").$myMpd->errStr,'error');

      xml_playlist($xml_root,$myMpd,true);
      xml_control($xml_root,$myMpd);
      break;

    case 'remove_webradio' :
      $ids = $_POST['edit'];
      rsort($ids);

      if (!send_queue_command($myMpd,$xml_root,MPD_CMD_PLREMOVE,$ids,false))
      	break;

      if (is_null($myMpd->PLDelete(WEBRADIOS_PLAYLIST_NAME)) ||
          is_null($myMpd->PLSave(WEBRADIOS_PLAYLIST_NAME)) )
        xml_message($xml_root,t("Unable to erase webradios : ").$myMpd->errStr,'error');

      // FIXME : there are a problem with the refresh after the SendCommandQueue
      // Must to be investigate
      $xml_root->new_child('javascript',"ajaxmp_ref.send_http_request('GET','command.php?action=get_webradios','',true);");
      break;

    case 'get_webradios' :
      xml_playlist($xml_root,$myMpd,true);
      xml_control($xml_root,$myMpd);
      break;

    case 'load_current_playlist' : 
      if (is_null($myMpd->PLClear()) || is_null($myMpd->PLLoad(CURRENT_PLAYLIST_NAME)))
        xml_message($xml_root,t("Unable to load the current playlist : ").$myMpd->errStr,'error');

      $pl = $xml_root->new_child('block',playlist());
      $pl->set_attribute('id','playlist');
      $pl = $xml_root->new_child('block',playlist_extra());
      $pl->set_attribute('id','playlist_extra');

      xml_playlist($xml_root,$myMpd);
      xml_control($xml_root,$myMpd);
      break;

    case 'shuffle_playlist' :
      if (is_null($myMpd->PLShuffle())) 
        xml_message($xml_root,t("Unable to shuffle the playlist : ").$myMpd->errStr,'error');

      xml_playlist($xml_root,$myMpd);
      xml_control($xml_root,$myMpd);
      break;

    case 'load_playlist' :
      $playlists = $_POST['edit'];

      if (!send_queue_command($myMpd,$xml_root,MPD_CMD_PLLOAD,$playlists,TRUE))
      	break;

      // FIXME : there are a problem with the refresh after the SendCommandQueue
      // Must to be investigate
      //xml_playlist($xml_root,$myMpd);
      //xml_control($xml_root,$myMpd);
      $xml_root->new_child('javascript',"ajaxmp_ref.send_http_request('GET','command.php?action=get_playlist','',true);");
      break;

    case 'save_playlist' :
      $playlist_name = format_text($_POST['edit'][0]);
      if (is_null($myMpd->PLSave($playlist_name))) 
        xml_message($xml_root,t("Unable to save the playlist : ").$myMpd->errStr,'error');
      else 
        xml_message($xml_root,t("The playlist has been saved"),'information');
      xml_playlist_dir($xml_root,$myMpd);
      break;

    case 'delete_playlist' :
      $playlists = $_POST['edit'];

      if (!send_queue_command($myMpd,$xml_root,MPD_CMD_PLDELETE,$playlists,TRUE))
      	break;

      // FIXME : there are a problem with the refresh after the SendCommandQueue
      // Must to be investigate
      //xml_playlist_dir($xml_root,$myMpd);
      $xml_root->new_child('javascript',"ajaxmp_ref.send_http_request('GET','command.php?action=playlist_dir','',true);");
      break;

    case 'move_pltrack' :
      if (is_null($myMpd->PLMoveTrack($_GET['id'],$_GET['new_id']))) 
        xml_message($xml_root,t("Unable to move the track : ").$myMpd->errStr,'error');
      xml_playlist($xml_root,$myMpd);
      xml_control($xml_root,$myMpd);
      break;

// Navigation Part
    case 'list_dir' :
      (isset($_GET['path']) ? xml_nav_dir($xml_root,$myMpd,format_text($_GET['path'])) : xml_nav_dir($xml_root,$myMpd));
      break;

    case 'playlist_dir' :
      xml_playlist_dir($xml_root,$myMpd);
      break;

    case 'webradios_dir':
      xml_webradios_dir($xml_root,$myMpd);
      break;

// Search Part
    case 'search' :
      $edit = $_POST['edit'];

      if (is_null($search_result = $myMpd->Search(urlencode($edit['search_type']),format_text($edit['search_string'])))) 
        xml_message($xml_root,t("Unable to do the search : ").$myMpd->errStr,'error');

      else {
        $xml_search = $xml_root->new_child('search_result','');

        foreach($search_result as $id => $entry) {
          $file = $xml_search->new_child('track_node',htmlspecialchars($entry["file"],ENT_QUOTES));
  	  $file->set_attribute("type","file");
	  }
        }
      break;

/***************************************************/
/******** Configuration Part ***********************/
/***************************************************/
    case 'config' :
      $xml_config = $xml_root->new_child('block',config_block($myMpd->xfade));
      $xml_config->set_attribute("id",'navigation_content');
      break;

    case 'update' :
      if (is_null($myMpd->DBRefresh()))
        xml_message($xml_root,t("Unable to update the database: ").$myMpd->errStr,'error');
      else
        xml_message($xml_root,t("The database has been updated."),'information');
      sleep(2);
      break;

    case 'set_xfade' : 
      $value = $_GET['value'];
      if (is_null($myMpd->Xfade($value)))
        xml_message($xml_root,t("Unable to set the new crossfadding: ").$myMpd->errStr,'error');
      else
        xml_message($xml_root,t("The new value of crossfading has been recorded."),'information');
      xml_control($xml_root,$myMpd);
      break;

    case 'record_config' :
      $config_value = array('default_library','lang','theme','enable_shortcut_keys','refresh','message_time','request_timeout','dbl_click');
      $edit = $_POST['edit'];
      $update = false;
      $errors = array();

      // Verif values before to record them
      foreach ($config_value as $value) {
        switch ($value) {
	  case 'refresh' :
	  case 'message_time' :
	  case 'request_timeout' :
	  case 'dbl_click' :
	    if (!is_numeric($edit[$value]))
	      $errors[$value] = t('You must enter a numeric value.');
	    break;
	  }
        }

      // If errors, report them
      if (!empty($errors)) {
        xml_form_errors($xml_root,$errors,'config_form');
	break;
	}

      foreach ($config_value as $value) {
        if (isset($edit[$value]) && $edit[$value] != $config->variable_get($value)) {
	  $config->variable_set($value,$edit[$value]);
	  if ($value == 'theme' || $value == 'lang') {
	    $update = true;
	    }
	  }
        }

      if ($update) {
      	$xml_root->new_child('javascript',"window.location.reload();");
	}
      else {
        xml_config($xml_root);
        xml_message($xml_root,t('The configuration has been saved.'),'information');
        $xml_config = $xml_root->new_child('block',config_block($myMpd->xfade));
        $xml_config->set_attribute("id",'navigation_content');
	}

      break;

// Navigation by artist and album
    case 'get_album' :
      $album = (isset($_GET['album']) ? format_text($_GET['album']) : NULL);
      $artist = (isset($_GET['artist']) ? format_text($_GET['artist']) : NULL);

      // FIXME : understand why the caracter "\" appear and try to correct this
      $artist = str_replace('\\','',$artist);

      // Remove HTML caracter
      $artist = html_entity_decode($artist,ENT_QUOTES);
      $album = html_entity_decode($album,ENT_QUOTES);

      $xml_tagnav = $xml_root->new_child('library','');
      $xml_tagnav->set_attribute('type','tag');

      if ($album) {
        // Find tracks of this album
        if (is_null($find_result = $myMpd->Find(MPD_SEARCH_ALBUM,$album))) 
          xml_message($xml_root,t("Unable to find track of this album : ").$myMpd->errStr,'error');
        else {
          foreach($find_result as $id => $entry) {
	    if (!$artist || $entry["Artist"]==$artist) {
              $file = $xml_tagnav->new_child('track_node',htmlspecialchars($entry["file"],ENT_QUOTES));
  	      $file->set_attribute("type","file");
	      }
	    }
          }
	}
      else {
        // Find tracks of this artist
        if ($artist) {
          if (is_null($find_result = $myMpd->Find(MPD_SEARCH_ARTIST,$artist))) 
            xml_message($xml_root,t("Unable to find track of this artist : ").$myMpd->errStr,'error');
          else {
            foreach($find_result as $id => $entry) {
              $file = $xml_tagnav->new_child('track_node',htmlspecialchars($entry["file"],ENT_QUOTES));
  	      $file->set_attribute("type","file");
	      }
            }
	  }
        }
      break;

    case 'get_artist' :
      $artist = (isset($_GET['artist']) ? format_text($_GET['artist']) : NULL);
      // Remove HTML caracter
      $artist = html_entity_decode($artist,ENT_QUOTES);

      $albums = $myMpd->GetAlbums($artist);
      if (is_null($albums)) {
        xml_message($xml_root,t("Unable to get the list of albums : ").$myMpd->errStr,'error');
        break;
	}

      $xml_tagnav = $xml_root->new_child('library','');
      $xml_tagnav->set_attribute('type','tag');
      xml_tag('albums',$xml_tagnav,$albums);
      
      // Find tracks of this artist
      if ($artist) {
        if (is_null($find_result = $myMpd->Find(MPD_SEARCH_ARTIST,$artist))) 
          xml_message($xml_root,t("Unable to find track of this artist : ").$myMpd->errStr,'error');
        else {
          foreach($find_result as $id => $entry) {
            $file = $xml_tagnav->new_child('track_node',htmlspecialchars($entry["file"],ENT_QUOTES));
  	    $file->set_attribute("type","file");
	    }
          }
	}
      break;

    case 'tag_navigation' :
      xml_tag_navigation($xml_root,$myMpd);
      break;

// first call at the server
    case 'init' :
      if (is_null($myMpd->playlist)) {
        xml_message($xml_root,t("Problem with the playlist :").$myMpd->errStr,'error');
        }
      else {
        if (isset($myMpd->playlist[0]) && preg_match("/^(http:)/",$myMpd->playlist[0]['file'])) {
          $pl = $xml_root->new_child('block',webradios_playlist());
          $pl->set_attribute('id','playlist');
          $pl = $xml_root->new_child('block',webradios_playlist_extra());
          $pl->set_attribute('id','playlist_extra');
          xml_playlist($xml_root,$myMpd,true);
          }
        else {
          switch ($config->variable_get('default_library')) {
	    case 'tag' :
              xml_tag_navigation($xml_root,$myMpd);
              break;
	    case 'playlist' :
              xml_playlist_dir($xml_root,$myMpd);
              break;
            case 'directory' :
	    default :
              xml_nav_dir($xml_root,$myMpd);
	      break;
	    }
          xml_playlist($xml_root,$myMpd,false);
          }
        }

      xml_control($xml_root,$myMpd);
      xml_volume($xml_root,$myMpd);
      xml_config($xml_root);
      xml_l10n($xml_root);
      break;

    default :
      xml_message($xml_root,t("Command not found : ").$_GET['action'],'error');
      break;
    }
  $myMpd->Disconnect();
  }

// we sent a xml document
header("Content-Type: text/xml");
// return the xml file at the client
print $xml->dump_mem();

?>
