<?php
// class.config.php

class configuration
{
	var $user_name;
	var $error;
	var $user_config;
	var $config_path = 'config/user_config.xml';

	function configuration($user,$default_config)
	{
		$this->user_name = $user;
		$this->user_config = $default_config;
	}

	function create_config_file()
	{
		// Verify the possibility to create the config file
		if (!$hd = @touch($this->config_path))
		{
			$this->error = t('Unable to create the file ').$this->config_path.'.';
			return false;
		}

		//initiate xml document
		$xml = domxml_new_doc('1.0');
		$xml->add_root('user_config');

		// record in the file
		$xml->dump_file($this->config_path);

		return true;
	}

	function load_config()
	{
		if ($this->user_name)
		{
			if (!file_exists($this->config_path))
			{
				if (!$this->create_config_file())
					return false;

				$_SESSION['config'] = array();
			}
			else
				$_SESSION['config'] = $this->get_user_config();
		}
		else
			$_SESSION['config'] = array();


		return true;
	}

	function variable_get($name)
	{
		return isset($_SESSION['config']) && is_array($_SESSION['config']) && array_key_exists($name,$_SESSION['config']) ?
			$_SESSION['config'][$name] : $this->user_config[$name];
	}

	function variable_set($name, $value)
	{
		$xml = domxml_open_file($this->config_path);
		$user = $this->get_user($xml);

		if ($user)
		{
			$config_parm = $user->get_elements_by_tagname($name);
			if (empty($config_parm))
			{
				$config_parm = $xml->create_element($name);
				$config_parm->set_content($value);
				$user->append_child($config_parm);
			}
			else
			{
				$config_text_node = $config_parm[0]->first_child();
				$config_text_node->set_content($value);
			}
		}
		else
		{
			$xml_root = $xml->document_element();

			$user = $xml->create_element('user');
			$user->set_attribute('name',$this->user_name);

			$config_parm = $xml->create_element($name);
			$config_parm->set_content($value);

			$user->append_child($config_parm);
			$xml_root->append_child($user);
		}
		// Save changes
		$xml->dump_file($this->config_path);

		$_SESSION['config'][$name] = $value;
	}

	function get_user_config()
	{
		$res = array();

		$xml = domxml_open_file($this->config_path);
		$user = $this->get_user($xml);

		if ($user && $user->has_child_nodes())
		{
			$config_parm = $user->first_child();
			do
			{
				$name = $config_parm->tagname();
				$value = $config_parm->get_content();
				$res[$name] = $value;
			} while ($config_parm = $config_parm->next_sibling());
		}

		return $res;
	}

	function get_user($xml)
	{
		$users = $xml->get_elements_by_tagname('user');

		foreach($users as $user)
		{
			if ($user->get_attribute('name') == $this->user_name)
				return $user;
		}
		return NULL;
	}

	function error()
	{
		return $this->error;
	}

}

function &init_config()
{
	global $user_config;

	$user = (isset($_SESSION['sess_user_id']) ? $_SESSION['sess_user_id'] : NULL);
	$config =& new configuration($user,$user_config);

	return $config;
}

?>
